describe('languages service tests', function() {
  var sandbox;
  var service;
  var observable;
  var http;
  
  beforeEach(function() {
    sandbox = sinon.sandbox.create();
    http = {
      get: function() {},
      post: function() {},
      delete: function() {}
    };
    
    service = new app.LanguagesService(http);
  
    observable = {
      map: function() {},
      catch: function() {}
    };
    
    sandbox.stub(http, 'get')
         .withArgs('/languages')
         .returns(observable);
  
    sandbox.stub(http, 'post')
         .withArgs('/languages')
         .returns(observable);

    sandbox.stub(observable, 'map')
         .withArgs(service.extractData)
         .returns(observable);
  
    sandbox.stub(observable, 'catch')
         .withArgs(service.returnError)
         .returns(observable);
  }); 
  
  afterEach(function() {
    sandbox.restore();
  });
  
  it('get should call http.get and register handlers', function() {
    expect(service.get()).to.be.eql(observable);
    expect(http.get.calledWith('/languages')).to.be.true;
    expect(observable.map.calledWith(service.extractData)).to.be.true;
    expect(observable.catch.calledWith(service.returnError)).to.be.true;
  });
  
  it('extractData should return json() response', function() {           
    var fakeData = '...stuff...';
    var fakeResponse = {status: 200, json: function() { return fakeData; }}
    
    expect(service.extractData(fakeResponse)).to.be.eql(fakeData);
  });
  
  it('extractData should throw exception of status not 200', function() {           
    var fakeResponse = {status: 404}
  
    var call = function() { service.extractData(fakeResponse); }
    
    expect(call).to.throw('Error, status: 404')
  });                                          
  
  it('returnError should call Observable.throw', function() {
    var error = {message: 'oops'};
    
    var observableMock = 
      sandbox.mock(Rx.Observable)
           .expects('throw')
           .withArgs('oops');
    
    service.returnError(error);
    observableMock.verify();
  });
  
  
  it('returnError should call Observable.throw with no error.message', function() {
    var error = {};
    
    var observableMock = 
      sandbox.mock(Rx.Observable)
           .expects('throw')
           .withArgs('Error: request failed');
    
    service.returnError(error);
    observableMock.verify();
  });
  
  it('should inject dependency', function() {
    var parameters = Reflect.getMetadata('parameters', app.LanguagesService);
    
    expect(parameters[0]).to.be.eql([ng.http.Http]);
  });
  
  it('add should call http.post and register handlers', function() {
    var language = {name: 'l1', author: 'a1'};
    var options = 
      {headers: new ng.http.Headers({'Content-Type': 'application/json'})};

    expect(service.add(language)).to.be.eql(observable);
    expect(http.post.calledWith('/languages', JSON.stringify(language), options)).to.be.true;
    expect(observable.map.calledWith(service.extractData)).to.be.true;
    expect(observable.catch.calledWith(service.returnError)).to.be.true;
  });

  it('extractData should return text if no json', function() {
    var fakeData = '...stuff...';
    var fakeResponse = {status: 200, text: function() { return fakeData; }}
    
    expect(service.extractData(fakeResponse)).to.be.eql(fakeData);
  });
   
  it('delete should call http.delete and register handlers', function() {
    var languageId = '1234';

    sandbox.stub(http, 'delete')
         .withArgs('/languages/' + languageId)
         .returns(observable);
    
    expect(service.delete(languageId)).to.be.eql(observable);
    expect(http.delete.calledWith('/languages/' + languageId)).to.be.true;
    expect(observable.map.calledWith(service.extractData)).to.be.true;
    expect(observable.catch.calledWith(service.returnError)).to.be.true;
  });
  
});